1 VERSIONINFO
FILEVERSION 1,5,4,0
PRODUCTVERSION 1,5,4,0
{
 BLOCK "StringFileInfo"
 {
  BLOCK "040904E4"
  {
   VALUE "Comments", "WinFF Video Converter\000"
   VALUE "CompanyName", "WinFF.org\000"
   VALUE "FileDescription", "WinFF Video Converter\000"
   VALUE "FileVersion", "1.5.4.0\000"
   VALUE "InternalName", "winff\000"
   VALUE "LegalCopyright", "Copyright 2014, GPL v3 by Matthew Weatherford\000"
   VALUE "LegalTrademarks", "\000"
   VALUE "OriginalFilename", "winff.exe\000"
   VALUE "ProductName", "WinFF\000"
   VALUE "ProductVersion", "1.5.4.0\000"
  }
 }
 BLOCK "VarFileInfo"
 {
  VALUE "Translation", 0x0409, 0x04E4
 }
}

#define RT_MANIFEST  24
#define CREATEPROCESS_MANIFEST_RESOURCE_ID 1
#define ISOLATIONAWARE_MANIFEST_RESOURCE_ID 2
#define ISOLATIONAWARE_NOSTATICIMPORT_MANIFEST_RESOURCE_ID 3


